@@ -6,6 +6,8 @@ This module stores the classes (Organism, Scaffold, Hit) used in cblaster.
 
 import re
 import json
+import itertools
+from abc import ABC, abstractmethod
 
 from cblaster.formatters import (
     binary,
@@ -15,18 +17,20 @@ from cblaster.formatters import (
 )
 
 
-class Serializer:
+class Serializer(ABC):
     """JSON serialisation mixin class.
 
     Classes that inherit from this class should implement `to_dict` and
     `from_dict` methods.
     """
 
+    @abstractmethod
     def to_dict(self):
         """Serialises class to dict."""
         raise NotImplementedError
 
     @classmethod
+    @abstractmethod
     def from_dict(self, d):
         """Loads class from dict."""
         raise NotImplementedError
@@ -68,13 +72,18 @@ class Session(Serializer):
         queries (list): Names of query sequences.
         params (dict): Search parameters.
         organisms (list): Organism objects created in a search.
+        sequences (dict): Query sequence translations
+        query (Cluster): cblaster Cluster object for query
     """
 
-    def __init__(self, queries=None, sequences=None, params=None, organisms=None):
+    def __init__(
+        self, queries=None, sequences=None, params=None, organisms=None, query=None
+    ):
         self.queries = queries if queries else []
         self.params = params if params else {}
         self.organisms = organisms if organisms else []
         self.sequences = sequences if sequences else {}
+        self.query = query
 
     def __add__(self, other):
         if not isinstance(other, Session):
@@ -83,6 +92,7 @@ class Session(Serializer):
             raise ValueError("Query sequences do not match")
         return Session(
             queries=self.queries,
+            query=self.query,
             sequences=self.sequences,
             params=self.params,
             organisms=self.organisms + other.organisms,
@@ -90,8 +100,8 @@ class Session(Serializer):
 
     def to_dict(self):
         return {
+            "query": self.query.to_dict(save_subjects=True),
             "queries": self.queries,
-            "sequences": self.sequences,
             "params": self.params,
             "organisms": [o.to_dict() for o in self.organisms],
         }
@@ -115,8 +125,8 @@ class Session(Serializer):
     @classmethod
     def from_dict(cls, d):
         return cls(
+            query=Cluster.from_dict(d["query"]),
             queries=d.get("queries", None),
-            sequences=d.get("sequences", None),
             params=d.get("params", None),
             organisms=[Organism.from_dict(o) for o in d.get("organisms", [])],
         )
@@ -238,19 +248,34 @@ class Scaffold(Serializer):
         After clusters are added they are sorted based on score
 
         Args:
-            subject_lists (list): a list of lists Subject objects that
-            form clusters
+            subject_lists (list): a list of lists of Subject objects that are
+            form a clusters
             query_sequence_order (list): list of sequences of the order in the query file, is
             only provided if the query has a meningfull order (gbk, embl files).
         """
         for subjects in subject_lists:
             indices = [self.subjects.index(subject) for subject in subjects]
             cluster = Cluster(
-                indices, subjects, query_sequence_order=query_sequence_order
+                indices,
+                subjects,
+                query_sequence_order=query_sequence_order,
             )
             self.clusters.append(cluster)
         self.clusters.sort(key=lambda x: x.score, reverse=True)
 
+    def remove_subject(self, subject):
+        """Safely remove a subject from a cluster by removing it from the cluster as well.
+
+        Args:
+            subject (Subject): cblaster Subject object
+        """
+        remove_index = self.subjects.index(subject)
+        for cluster in self.clusters:
+            cluster.remove_subject(subject, remove_index)
+            if len(cluster.subjects) == 0:
+                self.clusters.remove(cluster)
+        del self.subjects[remove_index]
+
     def summary(self, hide_headers=False, delimiter=None, decimals=4):
         return summarise_scaffold(
             self, decimals=decimals, hide_headers=hide_headers, delimiter=delimiter,
@@ -269,7 +294,7 @@ class Scaffold(Serializer):
         clusters = [None] * len(d["clusters"])
         for index, cluster in enumerate(d["clusters"]):
             cluster_subjects = [subjects[ix] for ix in cluster["indices"]]
-            clusters[index] = Cluster.from_dict(cluster, *cluster_subjects)
+            clusters[index] = Cluster.from_dict(cluster, subjects=cluster_subjects)
         return cls(accession=d["accession"], subjects=subjects, clusters=clusters)
 
 
@@ -281,24 +306,36 @@ class Cluster(Serializer):
         of the parent scaffold
         subjects (list): Subject objects that are in this cluster. Note:
         These are not serialised for this cluster
+        intermediate_genes (list):
         start (int): The start coordinate of the cluster on the parent scaffold
         end (int): The end coordinate of the cluster on the parent scaffold
+        number (int): number that is unique for each cluster in order to identify them
     """
 
+    NUMBER = itertools.count()
+
     def __init__(
         self,
         indices=None,
         subjects=None,
+        intermediate_genes=None,
         query_sequence_order=None,
         score=None,
         start=None,
         end=None,
+        number=None,
     ):
         self.indices = indices if indices else []
         self.subjects = subjects if subjects else []
+        self.intermediate_genes = intermediate_genes if intermediate_genes else []
         self.score = score if score else self.calculate_score(query_sequence_order)
-        self.start = start if start else self.subjects[0].start
-        self.end = end if end else self.subjects[-1].end
+        self.start = start
+        self.end = end
+        self.number = number if number is not None else next(self.NUMBER)
+
+        if self.subjects and not (self.start or self.end):
+            self.start = self.subjects[0].start
+            self.end = self.subjects[-1].end
 
     def __iter__(self):
         return iter(self.subjects)
@@ -306,6 +343,45 @@ class Cluster(Serializer):
     def __len__(self):
         return len(self.subjects)
 
+    def __eq__(self, other):
+        if not isinstance(other, Cluster):
+            raise NotImplementedError("Expected Cluster object")
+        return (set(self.subjects) == set(other.subjects)
+                and self.score == other.score)
+
+    def __hash__(self):
+        return hash(id(self))
+
+    @property
+    def sequences(self):
+        return {
+            subject.name: subject.sequence
+            for subject in self.subjects
+        }
+
+    @property
+    def names(self):
+        return [subject.name for subject in self.subjects]
+
+    def remove_subject(self, subject, scaffold_index):
+        """Safely remove a subject from a cluster.
+
+        This is important when subjects become empty when recomputing a session with different treshholds
+
+        Args:
+            subject (Subject): cblaster Subject object
+            scaffold_index (int): the index of the subject in the scaffold it is saved in
+        """
+        try:
+            remove_index = self.subjects.index(subject)
+        except ValueError:
+            return
+        for index, index_value in enumerate(self.indices):
+            if index_value > scaffold_index:
+                self.indices[index] -= 1
+        del self.indices[remove_index]
+        del self.subjects[remove_index]
+
     def __calculate_synteny_score(self, query_sequence_order):
         if not query_sequence_order:
             return 0
@@ -326,15 +402,31 @@ class Cluster(Serializer):
 
     def __calculate_bitscore(self):
         return sum(
-            max(hit.bitscore for hit in subject.hits) for subject in self.subjects
+            max(hit.bitscore for hit in subject.hits)
+            for subject in self.subjects
+            if subject.hits
         )
 
+    @property
+    def intermediate_start(self):
+        """The start of the cluster taking the intermediate genes into account"""
+        if not self.intermediate_genes:
+            return self.start
+        return min(*[s.start for s in self.intermediate_genes], self.start)
+
+    @property
+    def intermediate_end(self):
+        """The end of the cluster taking the intermediate genes into account"""
+        if not self.intermediate_genes:
+            return self.end
+        return max(*[s.end for s in self.intermediate_genes], self.end)
+
     def calculate_score(self, query_sequence_order=None):
         """Calculate the score of the current cluster
 
         The score is based on accumulated blastbitscore, total amount of hits against the
         query and a synteny score if query sequence order is provided. If there are multiple
-        hits in a subject the hit with the top bitscore is selected for the caclulation.
+        hits in a subject the hit with the top bitscore is selected for the calculation.
 
         Args:
             query_sequence_order (list): list of sequences of the order in the query file, is
@@ -346,22 +438,27 @@ class Cluster(Serializer):
         bitscore = self.__calculate_bitscore()
         return bitscore / 10000 + len(self.subjects) + synteny_score
 
-    def to_dict(self):
+    def to_dict(self, save_subjects=False):
         return {
             "indices": self.indices,
+            "subjects": [s.to_dict() for s in self.subjects] if save_subjects else [],
+            "intermediate_genes": [g.to_dict() for g in self.intermediate_genes],
             "score": self.score,
             "start": self.start,
             "end": self.end,
+            "number": self.number,
         }
 
     @classmethod
-    def from_dict(cls, d, *subjects):
+    def from_dict(cls, d, subjects=None):
         return cls(
             indices=d["indices"],
-            subjects=subjects,
+            subjects=subjects or [Subject.from_dict(d) for d in d["subjects"]],
+            intermediate_genes=[Subject.from_dict(d) for d in d["intermediate_genes"]],
             score=d["score"],
             start=d["start"],
             end=d["end"],
+            number=d["number"],
         )
 
 
@@ -381,57 +478,89 @@ class Subject(Serializer):
     """
 
     def __init__(
-        self, hits=None, name=None, ipg=None, start=None, end=None, strand=None
+        self,
+        id=None,
+        hits=None,
+        name=None,
+        ipg=None,
+        start=None,
+        end=None,
+        strand=None,
+        sequence=None,
     ):
+        self.id = id
         self.hits = hits if hits else []
         self.ipg = ipg
         self.name = name
         self.start = int(start) if start is not None else None
         self.end = int(end) if end is not None else None
         self.strand = strand
+        self.sequence = sequence
+
+    def __key(self):
+        # make equals behaviour of higher classes consistent with different instances
+        return (
+            self.id,
+            tuple(sorted(self.hits, key=lambda x: x.bitscore)),
+            self.ipg,
+            self.start,
+            self.end,
+            self.strand
+        )
 
     def __eq__(self, other):
         if not isinstance(other, Subject):
             raise NotImplementedError("Expected Subject object")
-        return (
-            set(self.hits) == set(other.hits)
-            and self.ipg == other.ipg
-            and self.start == other.start
-            and self.end == other.end
-            and self.strand == other.strand
-        )
+        return self.__key() == other.__key()
+
+    def __hash__(self):
+        return hash(self.__key())
 
     def to_dict(self):
         return {
+            "id": self.id,
             "hits": [hit.to_dict() for hit in self.hits],
             "name": self.name,
             "ipg": self.ipg,
             "start": self.start,
             "end": self.end,
             "strand": self.strand,
+            "sequence": self.sequence,
         }
 
     def values(self, decimals=4):
-        records = []
-        for hit in self.hits:
-            record = (
-                *hit.values(decimals),
-                str(self.start),
-                str(self.end),
-                self.strand,
-            )
-            records.append(record)
-        return records
+        start = str(self.start)
+        end = str(self.end)
+        strand = "+" if self.strand == 1 else "-"
+        # If hits, this was found during the search 
+        # Otherwise, it's an intermediate
+        if len(self.hits) > 0:
+            return [
+                (*hit.values(decimals), start, end, strand)
+                for hit in self.hits
+            ]
+        record = (
+            "intermediate",
+            self.name,
+            "-", "-",
+            "-", "-",
+            start,
+            end,
+            strand,
+        )
+        return [record]
 
     @classmethod
     def from_dict(cls, d):
         return cls(
+            id=d.get("id"),
             hits=[Hit.from_dict(h) for h in d["hits"]],
             name=d.get("name"),
             ipg=d.get("ipg"),
             start=d.get("start"),
             end=d.get("end"),
             strand=d.get("strand"),
+            sequence=d.get("sequence"),
         )
 
 
@@ -456,21 +585,21 @@ class Hit(Serializer):
 
         if "gb" in subject or "ref" in subject:
             subject = re.search(r"\|([A-Za-z0-9\._]+)\|", subject).group(1)
-
+        # Made id & Coverage a None type, hmmer does not have those values
         self.subject = subject
         self.bitscore = float(bitscore)
-        self.identity = float(identity)
-        self.coverage = float(coverage)
+        self.identity = float(identity) if identity is not None else None
+        self.coverage = float(coverage) if coverage is not None else None
         self.evalue = float(evalue)
 
     def __str__(self):
         return (
             f"Hit: {self.query} - {self.subject}:"
-            f" {self.identity:.2%}/{self.coverage:.2%}"
+            f" {self.identity}/{self.coverage:}"
         )
 
     def __key(self):
-        return (self.query, self.bitscore, self.identity, self.coverage, self.evalue)
+        return self.query, self.bitscore, self.identity, self.coverage, self.evalue
 
     def __hash__(self):
         return hash(self.__key())
@@ -498,10 +627,10 @@ class Hit(Serializer):
         return [
             self.query,
             self.subject,
-            f"{round(self.identity, decimals):g}",
-            f"{round(self.coverage, decimals):g}",
-            f"{self.evalue:.{decimals}g}",
-            f"{round(self.bitscore, decimals):g}",
+            f"{round(self.identity, decimals):g}" if self.identity is not None else "",
+            f"{round(self.coverage, decimals):g}" if self.coverage is not None else "",
+            f"{self.evalue:.{decimals}g}" if self.evalue is not None else "",
+            f"{round(self.bitscore, decimals):g}" if self.bitscore is not None else "",
         ]
 
     def to_dict(self):
