@@ -127,6 +127,7 @@ def cblaster(
     intermediate_gene_distance=5000,
     intermediate_max_clusters=100,
     testing=False,
+    dmnd_sensitivity='fast',
 ):
     """Run cblaster.
 
@@ -289,6 +290,8 @@ def cblaster(
                 max_evalue=max_evalue,
                 blast_file=blast_file,
                 cpus=cpus,
+                dmnd_sensitivity=dmnd_sensitivity,
+                hitlist_size=hitlist_size,
             )
             LOG.info(
                 "Found %i hits meeting score thresholds for local search", len(results)
@@ -399,6 +402,20 @@ def main():
     if args.debug:
         LOG.setLevel(logging.DEBUG)
 
+
+    if args.subcommand == "config":
+        if not args.email and not args.api_key:
+            LOG.info(
+                "No e-mail or API key specified; if this is your first time"
+                " running cblaster config, please make sure you provide one."
+            )
+        config.write_config_file(
+            email=args.email,
+            api_key=args.api_key,
+            max_tries=args.max_tries,
+        )
+        return
+
     # Set BioPython Entrez parameters
     set_entrez()
 
@@ -453,6 +470,7 @@ def main():
             intermediate_gene_distance=args.max_distance,
             intermediate_max_clusters=args.maximum_clusters,
             testing=args.testing,
+            dmnd_sensitivity=args.dmnd_sensitivity,
         )
 
     elif args.subcommand == "gui":
@@ -511,18 +529,5 @@ def main():
             testing=args.testing,
         )
 
-    elif args.subcommand == "config":
-        if not args.email and not args.api_key:
-            LOG.info(
-                "No e-mail or API key specified; if this is your first time"
-                " running cblaster config, please make sure you provide one."
-            )
-        config.write_config_file(
-            email=args.email,
-            api_key=args.api_key,
-            max_tries=args.max_tries,
-        )
-
-
 if __name__ == "__main__":
     main()
