@@ -91,7 +91,8 @@ def add_makedb_subparser(subparsers):
     makedb.add_argument(
         "paths",
         type=lambda x: full_path(x, os.R_OK),
-        help="Path/s to genome files to use when building local databases",
+        help="Path/s to genome files to use when building local databases (can be gzipped). "
+             "Alternatively, path to one .txt file with one genome file per line.",
         nargs="+",
     )
     makedb.add_argument(
@@ -338,9 +339,17 @@ def add_searching_group(search):
         "--hitlist_size",
         type=int,
         default=5000,
-        help="Maximum total hits to save from a remote BLAST search (def. 5000). Setting"
+        help="Maximum total hits to save from a local or remote BLAST search (def. 5000). Setting"
              " this value too low may result in missed hits/clusters."
     )
+    group.add_argument(
+        "-ds",
+        "--dmnd_sensitivity",
+        type=str,
+        choices=["fast", "mid", "sensitive", "more", "very", "ultra"],
+        default="fast",
+        help="Level of sensitivity to use in local DIAMOND searches (def. 'fast')"
+    )
 
 
 def add_clustering_group(search):
@@ -378,8 +387,8 @@ def add_clustering_group(search):
         "-per",
         "--percentage",
         type=int,
-        default=50,
-        help="Filter on %% of query genes needed to be present in cluster",
+        default=0,
+        help="Percentage of query genes required to be present in cluster",
     )
 
 
