@@ -293,13 +293,11 @@ def serve_html(data, chart="heatmap"):
             httpd.shutdown()
 
 
-def plot_session(session, output=None, sort_clusters=False, max_clusters=None, testing=False):
+def plot_session(session, output, sort_clusters=False, max_clusters=None, testing=False):
     data = get_data(session, sort_clusters, max_clusters=max_clusters)
     if output:
         LOG.info(f"Saving cblaster plot HTML to: {output}")
         save_html(data, output)
-        if not testing:
-            webbrowser.open(output)
     else:
         serve_html(data)
 
